# SPDX-License-Identifier: MPL-2.0
# Copyright 2020-2022 John Mille<john@compose-x.io>

"""Top-level package for ECS Files Composer."""

__author__ = """John Preston"""
__email__ = "john@compose-x.io"
__version__ = "2.0.0"
